"""
Management command for ensuring any pending group membership, ie memberships associated with a pending enterprise user,
are removed after 90 days.
"""

import logging
from datetime import timedelta

from django.core.management.base import BaseCommand

from enterprise.models import EnterpriseCustomer, EnterpriseGroupMembership
from enterprise.utils import localized_utcnow

log = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    Management command for ensuring any pending group membership, ie memberships associated with a pending enterprise
    user, are removed after 90 days. Optionally supply a ``--enterprise_customer`` arg to only run this command on
    a singular customer.

    Example usage:
        $ ./manage.py remove_expired_pending_group_memberships
    """
    help = 'Removes pending group memberships if they are older than 90 days.'

    def add_arguments(self, parser):
        parser.add_argument("-e", "--enterprise_customer")

    def handle(self, *args, **options):
        queryset = EnterpriseGroupMembership.all_objects.all()
        if enterprise_arg := options.get("enterprise_customer"):
            try:
                enterprise_customer = EnterpriseCustomer.objects.get(uuid=enterprise_arg)
                queryset = queryset.filter(group__enterprise_customer=enterprise_customer)
            except EnterpriseCustomer.DoesNotExist as exc:
                log.exception(f'Enterprise Customer: {enterprise_arg} not found')
                raise exc
        expired_memberships = queryset.filter(
            enterprise_customer_user=None,
            pending_enterprise_customer_user__isnull=False,
            created__lte=localized_utcnow() - timedelta(days=90)
        )
        for membership in expired_memberships:
            pecu_to_delete = membership.pending_enterprise_customer_user
            pecu_to_delete.delete()
            membership.refresh_from_db()
            # https://github.com/jazzband/django-model-utils/blob/master/model_utils/models.py#L133-L158
            membership.delete(soft=False)



# Generated by Django 4.2.15 on 2024-10-15 17:56

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import simple_history.models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('enterprise', '0222_alter_enterprisegroup_group_type_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='DefaultEnterpriseEnrollmentIntention',
            fields=[
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('is_removed', models.BooleanField(default=False)),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('content_type', models.CharField(choices=[('course', 'Course'), ('course_run', 'Course Run')], help_text='The type of content (e.g. a course vs. a course run).', max_length=127)),
                ('content_key', models.CharField(help_text='A course or course run that related users should be automatically enrolled into.', max_length=255)),
                ('enterprise_customer', models.ForeignKey(help_text='The customer for which this default enrollment will be realized.', on_delete=django.db.models.deletion.CASCADE, related_name='default_enrollment_intentions', to='enterprise.enterprisecustomer')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='HistoricalDefaultEnterpriseEnrollmentRealization',
            fields=[
                ('id', models.IntegerField(auto_created=True, blank=True, db_index=True, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('history_id', models.AutoField(primary_key=True, serialize=False)),
                ('history_date', models.DateTimeField()),
                ('history_change_reason', models.CharField(max_length=100, null=True)),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('intended_enrollment', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='enterprise.defaultenterpriseenrollmentintention')),
                ('realized_enrollment', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='enterprise.enterprisecourseenrollment')),
            ],
            options={
                'verbose_name': 'historical default enterprise enrollment realization',
                'verbose_name_plural': 'historical default enterprise enrollment realizations',
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': ('history_date', 'history_id'),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name='HistoricalDefaultEnterpriseEnrollmentIntention',
            fields=[
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('is_removed', models.BooleanField(default=False)),
                ('uuid', models.UUIDField(db_index=True, default=uuid.uuid4, editable=False)),
                ('content_type', models.CharField(choices=[('course', 'Course'), ('course_run', 'Course Run')], help_text='The type of content (e.g. a course vs. a course run).', max_length=127)),
                ('content_key', models.CharField(help_text='A course or course run that related users should be automatically enrolled into.', max_length=255)),
                ('history_id', models.AutoField(primary_key=True, serialize=False)),
                ('history_date', models.DateTimeField()),
                ('history_change_reason', models.CharField(max_length=100, null=True)),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('enterprise_customer', models.ForeignKey(blank=True, db_constraint=False, help_text='The customer for which this default enrollment will be realized.', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='enterprise.enterprisecustomer')),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'historical default enterprise enrollment intention',
                'verbose_name_plural': 'historical default enterprise enrollment intentions',
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': ('history_date', 'history_id'),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name='DefaultEnterpriseEnrollmentRealization',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('intended_enrollment', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='enterprise.defaultenterpriseenrollmentintention')),
                ('realized_enrollment', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='enterprise.enterprisecourseenrollment')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='defaultenterpriseenrollmentintention',
            name='realized_enrollments',
            field=models.ManyToManyField(through='enterprise.DefaultEnterpriseEnrollmentRealization', to='enterprise.enterprisecourseenrollment'),
        ),
    ]



# Generated by Django 3.2.22 on 2024-01-29 13:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('integrated_channel', '0031_alter_integratedchannelapirequestlogs_options'),
    ]

    operations = [
        migrations.AlterField(
            model_name='integratedchannelapirequestlogs',
            name='endpoint',
            field=models.URLField(max_length=255),
        ),
    ]



"""
Tests for the `edx-enterprise` decorators.
"""

import unittest
import warnings
from importlib import import_module
from unittest import mock
from urllib.parse import parse_qs, unquote, urlparse

import ddt
from faker import Factory as FakerFactory
from pytest import mark, raises

from django.conf import settings
from django.contrib.auth.models import AnonymousUser
from django.http import Http404
from django.test import RequestFactory
from django.urls import reverse

from enterprise.decorators import (
    deprecated,
    disable_for_loaddata,
    enterprise_login_required,
    force_fresh_session,
    ignore_warning,
)
from test_utils import get_magic_name, mock_view_function
from test_utils.factories import EnterpriseCustomerFactory, EnterpriseCustomerIdentityProviderFactory, UserFactory

FAKER = FakerFactory.create()


@ddt.ddt
@mark.django_db
class TestEnterpriseDecorators(unittest.TestCase):
    """
    Tests for enterprise decorators.
    """

    def setUp(self):
        """
        Set up test environment.
        """
        super().setUp()
        self.provider_id = FAKER.slug()  # pylint: disable=no-member
        self.uuid = FAKER.uuid4()  # pylint: disable=no-member
        self.customer = EnterpriseCustomerFactory(uuid=self.uuid)
        self.identity_provider = EnterpriseCustomerIdentityProviderFactory(
            provider_id=self.provider_id,
            enterprise_customer=self.customer
        )
        self.session_engine = import_module(settings.SESSION_ENGINE)

    def _prepare_request(self, url, user):
        """
        Prepare request for test.
        """
        request = RequestFactory().get(url)
        request.user = user
        session_key = request.COOKIES.get(settings.SESSION_COOKIE_NAME)
        request.session = self.session_engine.SessionStore(session_key)
        return request

    def test_deprecated(self):
        """
        Calling a deprecated function emits a warning.
        """

        def func():
            """ Function to be decorated. """

        with warnings.catch_warnings(record=True) as warning:
            warnings.simplefilter('always')
            deprecated('Yep!')(func)()
            assert len(warning) == 1
            assert issubclass(warning[0].category, DeprecationWarning)
            assert str(warning[0].message) == 'You called the deprecated function `func`. Yep!'

    def test_ignore_warning(self):
        """
        Emitted warnings from a function are ignored.
        """

        def func():
            """ Function to be decorated. """

        with warnings.catch_warnings(record=True) as warning:
            warnings.simplefilter('always')
            ignore_warning(DeprecationWarning)(func)()
            deprecated('Yep!')(func)()
            assert len(warning) == 0

    @ddt.data(True, False)
    def test_disable_for_loaddata(self, raw):
        signal_handler_mock = mock.MagicMock()
        signal_handler_mock.__name__ = get_magic_name("Irrelevant")
        wrapped_handler = disable_for_loaddata(signal_handler_mock)

        wrapped_handler(raw=raw)

        assert signal_handler_mock.called != raw

    @ddt.data(
        {},  # Missing required parameter `enterprise_uuid` arguments in kwargs
        {'enterprise_uuid': ''},  # Required parameter `enterprise_uuid` with empty value in kwargs.
        # pylint: disable=no-member
        {'enterprise_uuid': FAKER.uuid4()},  # Invalid value of `enterprise_uuid` in kwargs.
    )
    def test_enterprise_login_required_raises_404(self, kwargs):
        """
        Test that the decorator `enterprise_login_required` raises `Http404`
        error when called with invalid or missing arguments.
        """
        view_function = mock_view_function()
        enterprise_launch_url = reverse(
            'enterprise_course_run_enrollment_page',
            args=[self.customer.uuid, 'course-v1:edX+DemoX+Demo_Course'],
        )
        request = self._prepare_request(enterprise_launch_url, UserFactory(is_active=True))

        with raises(Http404):
            enterprise_login_required(view_function)(request, **kwargs)

    @ddt.data(
        ('idp-1', 'idp-1', True, 'idp-1'),
        (None, 'idp-1', True, 'idp-1'),
        ('fake_idp', 'idp-1', True, None),
        ('idp-1', 'idp-1', False, None),
        (None, None, False, None),
    )
    @ddt.unpack
    def test_enterprise_login_required_redirects_for_anonymous_users(
            self,
            tpa_hint_param,
            identity_provided_id,
            link_identity_provider,
            redirected_tpa_hint
    ):
        """
        Test that the decorator `enterprise_login_required` returns Http
        Redirect for anonymous users.
        """
        enterprise_customer = EnterpriseCustomerFactory(uuid=FAKER.uuid4())  # pylint: disable=no-member
        identity_provider = EnterpriseCustomerIdentityProviderFactory()

        if identity_provided_id:
            identity_provider.provider_id = identity_provided_id
            identity_provider.save()

        if link_identity_provider:
            identity_provider.enterprise_customer = enterprise_customer
            identity_provider.save()

        view_function = mock_view_function()
        course_id = 'course-v1:edX+DemoX+Demo_Course'
        enterprise_launch_url = reverse(
            'enterprise_course_run_enrollment_page',
            args=[enterprise_customer.uuid, course_id],
        )
        if tpa_hint_param:
            enterprise_launch_url = f'{enterprise_launch_url}?tpa_hint={tpa_hint_param}'

        request = self._prepare_request(enterprise_launch_url, AnonymousUser())
        response = enterprise_login_required(view_function)(
            request, enterprise_uuid=enterprise_customer.uuid, course_id=course_id
        )

        # Assert that redirect status code 302 is returned when an anonymous
        # user tries to access enterprise course enrollment page.
        assert response.status_code == 302
        assert 'new_enterprise_login%3Dyes' in response.url
        if redirected_tpa_hint:
            assert f'tpa_hint%3D{redirected_tpa_hint}' in response.url
        else:
            assert f'tpa_hint%3D{redirected_tpa_hint}' not in response.url

    def test_enterprise_login_required_redirects_for_anonymous_users_with_querystring(self):
        """
        Test that the decorator `enterprise_login_required` returns Http
        Redirect for anonymous users while keeping the format of query
        parameters unchanged.
        """
        view_function = mock_view_function()
        course_id = 'course-v1:edX+DemoX+Demo_Course'
        course_enrollment_url = reverse(
            'enterprise_course_run_enrollment_page',
            args=[self.customer.uuid, course_id],
        )
        querystring = 'catalog=dummy-catalog-uuid'
        course_enrollment_url = '{course_enrollment_url}?{querystring}'.format(
            course_enrollment_url=course_enrollment_url, querystring=querystring
        )
        request = self._prepare_request(course_enrollment_url, AnonymousUser())

        response = enterprise_login_required(view_function)(
            request, enterprise_uuid=self.customer.uuid, course_id=course_id
        )

        # Assert that redirect status code 302 is returned when an anonymous
        # user tries to access enterprise course enrollment page.
        assert response.status_code == 302

        # Now verify that the query parameters in the querystring of next url
        # are unchanged
        next_url = parse_qs(urlparse(response.url).query)['next'][0]
        next_url_querystring = unquote(urlparse(next_url).query)
        assert 'new_enterprise_login=yes' in next_url_querystring
        assert 'tpa_hint' in next_url_querystring
        assert querystring in next_url_querystring

    def test_enterprise_login_required(self):
        """
        Test that the enterprise login decorator calls the view function.

        Test that the decorator `enterprise_login_required` calls the view
        function when:
            1. `enterprise_uuid` is provided and corresponding enterprise
                customer exists in database.
            2. User making the request is authenticated.

        """
        view_function = mock_view_function()
        course_id = 'course-v1:edX+DemoX+Demo_Course'
        enterprise_launch_url = reverse(
            'enterprise_course_run_enrollment_page',
            args=[self.customer.uuid, course_id],
        )
        request = self._prepare_request(enterprise_launch_url, UserFactory(is_active=True))

        enterprise_login_required(view_function)(
            request, enterprise_uuid=self.customer.uuid, course_id=course_id
        )

        # Assert that view function was called.
        assert view_function.called

    def test_force_fresh_session_anonymous_user(self):
        """
        Test that the force_fresh_session decorator calls the
        decorated view the request is made by an unauthenticated
        user.
        """
        view_function = mock_view_function()
        course_id = 'course-v1:edX+DemoX+Demo_Course'
        enterprise_launch_url = reverse(
            'enterprise_course_run_enrollment_page',
            args=[self.customer.uuid, course_id],
        )
        request = self._prepare_request(enterprise_launch_url, AnonymousUser())

        force_fresh_session(view_function)(
            request, enterprise_uuid=self.customer.uuid, course_id=course_id
        )

        # Assert that view function was called and the session flag was set.
        assert view_function.called

    @mock.patch('enterprise.decorators.get_identity_provider')
    def test_force_fresh_session_no_sso_provider(self, mock_get_idp):
        """
        Test that the force_fresh_session decorator calls the view function
        when no sso provider is configured.
        """
        mock_get_idp.return_value = None
        view_function = mock_view_function()
        course_id = 'course-v1:edX+DemoX+Demo_Course'
        enterprise_launch_url = reverse(
            'enterprise_course_run_enrollment_page',
            args=[self.customer.uuid, course_id],
        )
        request = self._prepare_request(enterprise_launch_url, UserFactory(is_active=True))

        force_fresh_session(view_function)(
            request, enterprise_uuid=self.customer.uuid, course_id=course_id
        )

        # Assert that view function was called.
        assert view_function.called

    def test_force_fresh_session_param_received(self):
        """
        Test that the force_fresh_session decorator calls the view function
        if the session is fresh.
        """
        view_function = mock_view_function()
        course_id = 'course-v1:edX+DemoX+Demo_Course'
        enterprise_launch_url = reverse(
            'enterprise_course_run_enrollment_page',
            args=[self.customer.uuid, course_id],
        )
        enterprise_launch_url += '?new_enterprise_login=yes'
        request = self._prepare_request(enterprise_launch_url, UserFactory(is_active=True))

        force_fresh_session(view_function)(
            request, enterprise_uuid=self.customer.uuid, course_id=course_id
        )

        # Assert that view function was called.
        assert view_function.called

    @mock.patch('enterprise.decorators.get_identity_provider')
    def test_force_fresh_session_param_not_received(self, mock_get_identity_provider):
        """
        Test that the force_fresh_session decorator redirects authenticated
        users with the appropriate provider config depending on the IdPs configuration.
        """
        mock_get_identity_provider.return_value.configure_mock(
            provider_id=self.provider_id,
        )
        view_function = mock_view_function()
        course_id = 'course-v1:edX+DemoX+Demo_Course'
        url_path = reverse(
            'enterprise_course_run_enrollment_page',
            args=[self.customer.uuid, course_id],
        )
        query = 'foo=bar'
        # Adding query parameter here to verify
        # the redirect URL is getting escaped properly.
        url = '{path}?{query}'.format(path=url_path, query=query)
        request = self._prepare_request(url, UserFactory(is_active=True))

        response = force_fresh_session(view_function)(
            request, enterprise_uuid=self.customer.uuid, course_id=course_id
        )

        # Assert that redirect status code 302 is returned
        assert response.status_code == 302
        # Assert the redirect URL query string is intact.
        redirect_url_query = parse_qs(urlparse(response.url).query)
        assert urlparse(unquote(redirect_url_query['redirect_url'][0])).query == query



# Generated by Django 3.2.23 on 2024-09-09 16:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sap_success_factors', '0023_auto_20240909_1556'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='key',
        ),
        migrations.RemoveField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='secret',
        ),
    ]



"""
Class for transmitting content metadata to Cornerstone.
"""
from integrated_channels.cornerstone.client import CornerstoneAPIClient
from integrated_channels.integrated_channel.transmitters.content_metadata import ContentMetadataTransmitter


class CornerstoneContentMetadataTransmitter(ContentMetadataTransmitter):
    """
    This transmitter prepares content metadata to be consumed by Cornerstone.
    """

    def __init__(self, enterprise_configuration, client=CornerstoneAPIClient):
        super().__init__(
            enterprise_configuration=enterprise_configuration,
            client=client
        )

    def transmit(self, create_payload, update_payload, delete_payload):
        """
        Dummy transmit method overriding base transmissions, intended to cause any normal invocations of the Cornerstone
        transmitter to result in a NOOP because Cornerstone is a pull based system and as such shouldn't send (push)
        information like other channels
        """
        self._log_info(
            f"Cornerstone base transmission invoked for config: {self.enterprise_configuration.id}. Treating as a NOOP"
        )
        pass  # pylint: disable=unnecessary-pass

    def transmit_for_web(self, create_payload, update_payload, delete_payload):
        """
        Alternative method to invoke a transmission for a Cornerstone channel.
        """
        return super().transmit(
            create_payload=create_payload, update_payload=update_payload, delete_payload=delete_payload
        )

    def _transmit_action(self, content_metadata_item_map, client_method, action_name):
        """
        Set status to IsActive to False for items that should be deleted.
        """
        if action_name == 'delete':
            for _, item in content_metadata_item_map.items():
                item.channel_metadata['IsActive'] = False
                item.save()
        return super()._transmit_action(content_metadata_item_map, client_method, action_name)



# Generated by Django 4.2.15 on 2024-09-02 08:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('canvas', '0037_canvasenterprisecustomerconfiguration_copy_id_and_secret_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='canvasenterprisecustomerconfiguration',
            name='client_id',
            field=models.CharField(blank=True, default='', help_text='The API Client ID provided to edX by the enterprise customer to be used to make API calls to Canvas on behalf of the customer.', max_length=255, null=True, verbose_name='API Client ID'),
        ),
        migrations.AlterField(
            model_name='canvasenterprisecustomerconfiguration',
            name='client_secret',
            field=models.CharField(blank=True, default='', help_text='The API Client Secret provided to edX by the enterprise customer to be used to make  API calls to Canvas on behalf of the customer.', max_length=255, null=True, verbose_name='API Client Secret'),
        ),
    ]



# Generated by Django 3.2.20 on 2023-11-16 18:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('moodle', '0027_alter_historicalmoodleenterprisecustomerconfiguration_options'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='enable_incomplete_progress_transmission',
            field=models.BooleanField(default=False, help_text='When set to True, the configured customer will receive learner data transmissions, for incomplete courses as well'),
        ),
        migrations.AddField(
            model_name='moodleenterprisecustomerconfiguration',
            name='enable_incomplete_progress_transmission',
            field=models.BooleanField(default=False, help_text='When set to True, the configured customer will receive learner data transmissions, for incomplete courses as well'),
        ),
    ]



# Generated by Django 5.2 on 2025-08-28 07:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0240_alter_enterprisecustomer_enable_data_sharing_consent_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='defaultenterpriseenrollmentintention',
            name='realized_enrollments',
            field=models.ManyToManyField(through='enterprise.DefaultEnterpriseEnrollmentRealization', through_fields=('intended_enrollment', 'realized_enrollment'), to='enterprise.enterprisecourseenrollment'),
        ),
    ]



# Generated by Django 3.2.12 on 2022-03-23 17:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('moodle', '0011_auto_20220302_2231'),
    ]

    operations = [
        migrations.AlterField(
            model_name='moodlelearnerdatatransmissionaudit',
            name='completed_timestamp',
            field=models.CharField(blank=True, help_text='Represents the Moodle representation of a timestamp: yyyy-mm-dd, which is always 10 characters. Can be left unset for audit transmissions.', max_length=10, null=True),
        ),
    ]



# Generated by Django 3.2.15 on 2023-01-03 14:44

from django.db import connection, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('degreed', '0026_move_and_recrete_completed_timestamp'),
    ]

    db_engine = connection.settings_dict['ENGINE']
    if 'sqlite3' in db_engine:
        operations = [
            migrations.AlterIndexTogether(
                name='degreedlearnerdatatransmissionaudit',
                index_together={('enterprise_customer_uuid', 'plugin_configuration_id')},
            ),
        ]
    elif 'postgresql' in db_engine:
        operations = [
            migrations.SeparateDatabaseAndState(
                state_operations=[
                    migrations.AlterIndexTogether(
                        name='degreedlearnerdatatransmissionaudit',
                        index_together={('enterprise_customer_uuid', 'plugin_configuration_id')},
                    ),
                ],
                database_operations=[
                    migrations.RunSQL(sql="""
                        CREATE INDEX degreed_dldta_85936b55_idx
                        ON degreed_degreedlearnerdatatransmissionaudit (enterprise_customer_uuid, plugin_configuration_id)
                    """, reverse_sql="""
                        DROP INDEX degreed_dldta_85936b55_idx
                    """),
                ]
            ),
        ]
    elif 'mysql' in db_engine:
        operations = [
            migrations.SeparateDatabaseAndState(
                state_operations=[
                    migrations.AlterIndexTogether(
                        name='degreedlearnerdatatransmissionaudit',
                        index_together={('enterprise_customer_uuid', 'plugin_configuration_id')},
                    ),
                ],
                database_operations=[
                    migrations.RunSQL(sql="""
                        CREATE INDEX degreed_dldta_85936b55_idx
                        ON degreed_degreedlearnerdatatransmissionaudit (enterprise_customer_uuid, plugin_configuration_id)
                        ALGORITHM=INPLACE LOCK=NONE
                    """, reverse_sql="""
                        DROP INDEX degreed_dldta_85936b55_idx
                        ON degreed_degreedlearnerdatatransmissionaudit
                    """),
                ]
            ),
        ]
    else:
        operations = [
            migrations.AlterIndexTogether(
                name='degreedlearnerdatatransmissionaudit',
                index_together={('enterprise_customer_uuid', 'plugin_configuration_id')},
            ),
        ]



# Generated by Django 3.2.21 on 2023-10-06 09:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0190_auto_20231003_0719'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='chatgptresponse',
            options={'verbose_name': 'ChatGPT Response', 'verbose_name_plural': 'ChatGPT Responses'},
        ),
        migrations.AddField(
            model_name='chatgptresponse',
            name='prompt_type',
            field=models.CharField(choices=[('learner_progress', 'Learner progress'), ('learner_engagement', 'Learner engagement')], help_text='Prompt type.', max_length=32, null=True),
        ),
    ]



# Generated by Django 3.2.19 on 2024-02-20 17:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('canvas', '0033_canvaslearnerdatatransmissionaudit_canvas_unique_enrollment_course_id'),
    ]

    operations = [
        migrations.DeleteModel(
            name='HistoricalCanvasEnterpriseCustomerConfiguration',
        ),
    ]



"""
Enterprise Integrated Channel Django application initialization.
"""

from django.apps import AppConfig


class IntegratedChannelConfig(AppConfig):
    """
    Configuration for the Enterprise Integrated Channel Django application.
    """
    name = 'integrated_channels.integrated_channel'
    verbose_name = "Enterprise Integrated Channels"

    # TODO: We should move these integrated-channel-specific retirement handlers to the edx-integrated-channels repo
    # and use `channel_integrations.integrated_channel` instead of `integrated_channels.integrated_channel`
    def ready(self):
        """
        Perform one-time initialization: connect signal handlers.
        """
        import integrated_channels.integrated_channel.signals  # noqa: F401  pylint: disable=import-outside-toplevel,unused-import



# Generated by Django 2.2.19 on 2021-03-08 18:45

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('enterprise', '0124_auto_20210301_1309'),
    ]

    operations = [
        migrations.CreateModel(
            name='UpdateRoleAssignmentsWithCustomersConfig',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('change_date', models.DateTimeField(auto_now_add=True, verbose_name='Change date')),
                ('enabled', models.BooleanField(default=False, verbose_name='Enabled')),
                ('role', models.CharField(blank=True, choices=[('enterprise_admin', 'enterprise_admin'), ('enterprise_learner', 'enterprise_learner'), ('enterprise_catalog_admin', 'enterprise_catalog_admin'), ('enterprise_openedx_operator', 'enterprise_openedx_operator')], help_text='Specifies which user role assignments to update.  If unspecified, will update for all roles.', max_length=100)),
                ('batch_size', models.IntegerField(default=500, help_text='Number of user role asssignments to update in each batch of updates.')),
                ('enterprise_customer_uuid', models.CharField(blank=True, help_text='The enterprise customer to limit role assignments to.', max_length=36)),
                ('dry_run', models.BooleanField(default=True, help_text='If set, no updates or creates will occur; will instead iterate over the assignments that would be modified or created')),
                ('changed_by', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='Changed by')),
            ],
        ),
    ]



"""
Tests for the `edx-enterprise` models module.
"""

import unittest
from unittest import mock

from pytest import mark

from django.contrib import auth
from django.db.models.signals import pre_migrate

import enterprise
import integrated_channels
from test_utils.factories import EnterpriseCustomerFactory, UserFactory

User = auth.get_user_model()


@mark.django_db
class TestEnterpriseConfig(unittest.TestCase):
    """
    Test edx-enterprise app config.
    """

    def setUp(self):
        """
        Set up test environment.
        """
        super().setUp()
        self.post_save_mock = mock.Mock()
        patcher = mock.patch('enterprise.signals.handle_user_post_save', self.post_save_mock)
        patcher.start()
        self.app_config = enterprise.apps.EnterpriseConfig('enterprise', enterprise)
        self.addCleanup(patcher.stop)

    def test_ready_connects_user_post_save_handler(self):
        self.app_config.ready()

        user = UserFactory()

        assert self.post_save_mock.call_count == 1
        call_args, call_kwargs = self.post_save_mock.call_args_list[0]
        assert call_args == ()
        assert call_kwargs["sender"] == User
        assert call_kwargs["instance"] == user
        assert call_kwargs["created"]

    def test_ready_does_not_fire_user_post_save_handler_for_other_models(self):
        self.app_config.ready()
        EnterpriseCustomerFactory()

        assert not self.post_save_mock.called

    def test_ready_disconnects_user_post_save_handler_for_migration(self):
        self.app_config.ready()
        pre_migrate.send(mock.Mock())

        UserFactory()

        assert not self.post_save_mock.called


@mark.django_db
class TestIntegratedChannelConfig(unittest.TestCase):
    """
    Test integrated_channels.integrated_channel app config.
    """

    def setUp(self):
        """
        Set up test environment
        """
        super().setUp()
        self.app_config = integrated_channels.integrated_channel.apps.IntegratedChannelConfig(
            'integrated_channel', integrated_channels.integrated_channel
        )

    def test_name(self):
        assert self.app_config.name == 'integrated_channel'


@mark.django_db
class TestSAPSuccessFactorsConfig(unittest.TestCase):
    """
    Test integrated_channels.sap_success_factors app config.
    """

    def setUp(self):
        """
        Set up test environment
        """
        super().setUp()
        self.app_config = integrated_channels.sap_success_factors.apps.SAPSuccessFactorsConfig(
            'sap_success_factors', integrated_channels.sap_success_factors
        )

    def test_name(self):
        assert self.app_config.name == 'sap_success_factors'


@mark.django_db
class TestDegreedConfig(unittest.TestCase):
    """
    Test integrated_channels.degreed app config.
    """

    def setUp(self):
        """
        Set up test environment
        """
        super().setUp()
        self.app_config = integrated_channels.degreed.apps.DegreedConfig(
            'degreed', integrated_channels.degreed
        )

    def test_name(self):
        assert self.app_config.name == 'degreed'


@mark.django_db
class TestCanvasConfig(unittest.TestCase):
    """
    Test integrated_channels.canvas app config.
    """

    def setUp(self):
        """
        Set up test environment
        """
        super().setUp()
        self.app_config = integrated_channels.canvas.apps.CanvasConfig(
            'canvas', integrated_channels.canvas
        )

    def test_name(self):
        assert self.app_config.name == 'canvas'


@mark.django_db
class TestBlackboardConfig(unittest.TestCase):
    """
    Test integrated_channels.blackboard app config.
    """

    def setUp(self):
        """
        Set up test environment
        """
        super().setUp()
        self.app_config = integrated_channels.blackboard.apps.BlackboardConfig(
            'blackboard', integrated_channels.blackboard
        )

    def test_name(self):
        assert self.app_config.name == 'blackboard'


@mark.django_db
class TestXAPIConfig(unittest.TestCase):
    """
    Test integrated_channels.xapi app config.
    """

    def setUp(self):
        """
        Set up test environment
        """
        super().setUp()
        self.app_config = integrated_channels.xapi.apps.XAPIConfig(
            'xapi', integrated_channels.xapi
        )

    def test_name(self):
        assert self.app_config.name == 'xapi'


@mark.django_db
class TestMoodleConfig(unittest.TestCase):
    """
    Test integrated_channels.moodle app config.
    """

    def setUp(self):
        """
        Set up test environment
        """
        super().setUp()
        self.app_config = integrated_channels.moodle.apps.MoodleConfig(
            'moodle', integrated_channels.moodle
        )

    def test_name(self):
        assert self.app_config.name == 'moodle'



"""
Tests for the `edx-enterprise` tpa-pipeline module.
"""

import unittest
from unittest import mock

import ddt
from pytest import mark

from django.contrib.messages.storage import fallback
from django.contrib.sessions.backends import cache
from django.test import RequestFactory

from enterprise.models import EnterpriseCustomerUser
from enterprise.tpa_pipeline import get_enterprise_customer_for_running_pipeline, handle_enterprise_logistration
from test_utils.factories import (
    EnterpriseCustomerFactory,
    EnterpriseCustomerIdentityProviderFactory,
    EnterpriseCustomerSsoConfigurationFactory,
    UserFactory,
)


@ddt.ddt
@mark.django_db
class TestTpaPipeline(unittest.TestCase):
    """
    Test functions in the tpa_pipeline module.
    """

    def setUp(self):
        ecidp = EnterpriseCustomerIdentityProviderFactory(provider_id='provider_slug')
        self.customer = ecidp.enterprise_customer
        self.user = UserFactory(is_active=True)
        self.request_factory = RequestFactory()
        self.request = self.request_factory.get('/')
        self.request.session = cache.SessionStore()
        super().setUp()

    def get_mocked_sso_backend(self):
        """
        Get a mocked request backend with mocked strategy.
        """
        # Monkey-patch storage for messaging;   pylint: disable=protected-access
        self.request._messages = fallback.FallbackStorage(self.request)

        strategy_mock = mock.MagicMock(request=self.request)
        backend = mock.MagicMock(
            name=None,
            strategy=strategy_mock,
        )
        return backend

    @ddt.data(False, True)
    def test_handle_enterprise_logistration_user_linking(
            self,
            user_is_active,
    ):
        """
        Test that we create an EnterpriseCustomerUser, then return.
        """
        backend = self.get_mocked_sso_backend()
        self.user = UserFactory(is_active=user_is_active)
        with mock.patch('enterprise.tpa_pipeline.get_enterprise_customer_for_running_pipeline') as fake_get_ec:
            with mock.patch('enterprise.tpa_pipeline.get_sso_provider') as fake_get_sso_provider:
                with mock.patch('enterprise.tpa_pipeline.validate_provider_config'):
                    enterprise_customer = EnterpriseCustomerFactory(
                        enable_data_sharing_consent=False
                    )
                    provider_config = EnterpriseCustomerIdentityProviderFactory(enterprise_customer=enterprise_customer)
                    fake_get_sso_provider.return_value = provider_config.provider_id

                    fake_get_ec.return_value = enterprise_customer
                    assert handle_enterprise_logistration(backend, self.user) is None
                    assert EnterpriseCustomerUser.objects.filter(
                        enterprise_customer=enterprise_customer,
                        user_id=self.user.id,
                        active=True
                    ).count() == 1

    def test_handle_enterprise_logistration_not_user_linking(self):
        """
        Test if there is not any enterprise customer then EnterpriseCustomerUser would not be associated with it.
        """
        backend = self.get_mocked_sso_backend()
        self.user = UserFactory()
        with mock.patch('enterprise.tpa_pipeline.get_enterprise_customer_for_running_pipeline') as fake_get_ec:
            with mock.patch('enterprise.tpa_pipeline.get_sso_provider') as fake_get_sso_provider:
                enterprise_customer = EnterpriseCustomerFactory(
                    enable_data_sharing_consent=False
                )
                fake_get_ec.return_value = None
                fake_get_sso_provider.return_value = None
                assert handle_enterprise_logistration(backend, self.user) is None
                assert EnterpriseCustomerUser.objects.filter(
                    enterprise_customer=enterprise_customer,
                    user_id=self.user.id,
                    active=True
                ).count() == 0

    def test_handle_enterprise_logistration_user_multiple_enterprises_linking(self):
        """
        Test that if user has multiple enterprise_customers then active status of latest
         enterprise_customer with which user is logged in will be marked as True and active
          status of other enterprise_customers will be marked as False.
        """
        backend = self.get_mocked_sso_backend()
        self.user = UserFactory(is_active=True)
        with mock.patch('enterprise.tpa_pipeline.get_enterprise_customer_for_running_pipeline') as fake_get_ec:
            with mock.patch('enterprise.tpa_pipeline.get_sso_provider') as fake_get_sso_provider:
                with mock.patch('enterprise.tpa_pipeline.validate_provider_config'):
                    enterprise_customer = EnterpriseCustomerFactory(
                        enable_data_sharing_consent=False
                    )
                    enterprise_customer_old = EnterpriseCustomerFactory(
                        enable_data_sharing_consent=False
                    )
                    EnterpriseCustomerUser.objects.create(
                        enterprise_customer=enterprise_customer_old,
                        user_id=self.user.id,
                        active=True
                    )
                    fake_get_ec.return_value = enterprise_customer
                    fake_get_sso_provider.return_value = 'test-sso-provider-id'
                    assert handle_enterprise_logistration(backend, self.user) is None
                    assert EnterpriseCustomerUser.objects.filter(
                        enterprise_customer=enterprise_customer,
                        user_id=self.user.id,
                        active=True
                    ).count() == 1
                    assert EnterpriseCustomerUser.objects.filter(
                        enterprise_customer=enterprise_customer_old,
                        user_id=self.user.id,
                        active=False
                    ).count() == 1

    @ddt.data(
        (False, 'facebook'),
        (True, 'facebook'),
        (True, 'google-oauth2'),
        (False, 'google-oauth2'),
    )
    @ddt.unpack
    def test_social_auth_user_login_associated_with_multiple_enterprise(self,
                                                                        new_association,
                                                                        backend_name):
        """
        Test redirect to enterprise selection page, if socialAuth user has LMS attached account
        and part of multiple enterprises
        """
        kwargs = {'new_association': new_association}
        backend = self.get_mocked_sso_backend()
        backend.name = backend_name
        backend.strategy.session_get.return_value = 'not-an-enrollment-url'
        self.user = UserFactory(is_active=True)
        enterprise_customer = EnterpriseCustomerFactory(
            enable_data_sharing_consent=False
        )
        enterprise_customer_old = EnterpriseCustomerFactory(
            enable_data_sharing_consent=False
        )
        EnterpriseCustomerUser.objects.create(
            enterprise_customer=enterprise_customer_old,
            user_id=self.user.id,
            active=False
        )
        EnterpriseCustomerUser.objects.create(
            enterprise_customer=enterprise_customer,
            user_id=self.user.id,
            active=True
        )
        with mock.patch('enterprise.tpa_pipeline.get_enterprise_customer_for_running_pipeline') as fake_get_ec:
            with mock.patch('enterprise.tpa_pipeline.select_enterprise_page_as_redirect_url') as ent_page_redirect:
                with mock.patch('enterprise.tpa_pipeline.get_sso_provider') as fake_get_sso_provider:
                    fake_get_sso_provider.return_value = None
                    fake_get_ec.return_value = None
                    handle_enterprise_logistration(backend, self.user, **kwargs)
                    if new_association:
                        ent_page_redirect.assert_not_called()
                    else:
                        ent_page_redirect.assert_called_once()

    @ddt.data(
        (False, 'facebook'),
        (False, 'google-oauth2'),
    )
    @ddt.unpack
    def test_social_auth_user_login_associated_with_one_enterprise(self, new_association, backend_name):
        """
        Test that if socialAuth user has edx attached account and is part of one enterprises then redirection url
        is not changed
        """
        kwargs = {'new_association': new_association}
        backend = self.get_mocked_sso_backend()
        backend.name = backend_name
        backend.strategy.session_get.return_value = 'not-an-enrollment-url'
        self.user = UserFactory(is_active=True)
        enterprise_customer = EnterpriseCustomerFactory(
            enable_data_sharing_consent=False
        )

        EnterpriseCustomerUser.objects.create(
            enterprise_customer=enterprise_customer,
            user_id=self.user.id,
            active=False
        )
        with mock.patch('enterprise.tpa_pipeline.get_enterprise_customer_for_running_pipeline') as fake_get_ec:
            with mock.patch('enterprise.tpa_pipeline.select_enterprise_page_as_redirect_url') as ent_page_redirect:
                with mock.patch('enterprise.tpa_pipeline.get_sso_provider') as fake_get_sso_provider:
                    fake_get_sso_provider.return_value = None
                    fake_get_ec.return_value = None
                    handle_enterprise_logistration(backend, self.user, **kwargs)
                    ent_page_redirect.assert_not_called()

    @ddt.data(
        (True, False, 'facebook'),
        (False, False, 'facebook'),
        (True, True, 'facebook'),
        (False, True, 'facebook'),
        (True, True, 'google-oauth2'),
        (False, True, 'google-oauth2'),
        (True, False, 'google-oauth2'),
        (False, False, 'google-oauth2'),
    )
    @ddt.unpack
    def test_bypass_enterprise_selection_page_for_enrollment_url_login(self,
                                                                       using_enrollment_url,
                                                                       new_association,
                                                                       backend_name):
        """
        Test that enterprise selection page is bypassed if socialAuth user is part of multiple enterprises
        and uses an enrollment url for login
        """
        kwargs = {'new_association': new_association}
        backend = self.get_mocked_sso_backend()
        backend.name = backend_name
        if using_enrollment_url:
            backend.strategy.session_get.return_value = '/enterprise/12e87171-fb0a/course/course-v1:Test/enroll'
        else:
            backend.strategy.session_get.return_value = 'not-an-enrollment-url'
        self.user = UserFactory(is_active=True)
        enterprise_customer = EnterpriseCustomerFactory(
            enable_data_sharing_consent=False
        )
        enterprise_customer_old = EnterpriseCustomerFactory(
            enable_data_sharing_consent=False
        )
        EnterpriseCustomerUser.objects.create(
            enterprise_customer=enterprise_customer_old,
            user_id=self.user.id,
            active=False
        )
        EnterpriseCustomerUser.objects.create(
            enterprise_customer=enterprise_customer,
            user_id=self.user.id,
            active=True
        )
        with mock.patch('enterprise.tpa_pipeline.get_enterprise_customer_for_running_pipeline') as fake_get_ec:
            with mock.patch(
                    'enterprise.tpa_pipeline.select_enterprise_page_as_redirect_url') as ent_page_redirect:
                with mock.patch('enterprise.tpa_pipeline.get_sso_provider') as fake_get_sso_provider:
                    fake_get_sso_provider.return_value = None
                    fake_get_ec.return_value = None
                    handle_enterprise_logistration(backend, self.user, **kwargs)
                    if new_association or using_enrollment_url:
                        ent_page_redirect.assert_not_called()
                    else:
                        ent_page_redirect.assert_called_once()

    def test_get_ec_for_pipeline(self):
        """
        Test that we get the correct results for a given running pipeline.
        """
        with mock.patch('enterprise.tpa_pipeline.Registry') as fake_registry:
            provider = mock.MagicMock(provider_id='provider_slug')
            fake_registry.get_from_pipeline.return_value = provider
            assert get_enterprise_customer_for_running_pipeline(self.request, 'pipeline') == self.customer

            # pipeline is None
            assert get_enterprise_customer_for_running_pipeline(self.request, None) is None

            # provider_id is None
            provider = mock.MagicMock(provider_id=None)
            fake_registry.get_from_pipeline.return_value = provider
            assert get_enterprise_customer_for_running_pipeline(self.request, 'pipeline') is None

    def test_enterprise_logistration_validates_sso_orchestration_config(self):
        """
        Test that an enterprise logistration flow validates the customer's sso integration config.
        """
        backend = self.get_mocked_sso_backend()
        self.user = UserFactory(is_active=True)
        with mock.patch('enterprise.tpa_pipeline.get_enterprise_customer_for_running_pipeline') as fake_get_ec:
            with mock.patch('enterprise.tpa_pipeline.get_sso_provider') as fake_get_sso_provider:
                enterprise_customer = EnterpriseCustomerFactory(
                    enable_data_sharing_consent=False
                )
                customer_sso_integration_config = EnterpriseCustomerSsoConfigurationFactory(
                    enterprise_customer=enterprise_customer,
                    validated_at=None,
                )
                customer_sso_integration_config.save()
                EnterpriseCustomerUser.objects.create(
                    enterprise_customer=enterprise_customer,
                    user_id=self.user.id,
                    active=True,
                )
                fake_get_ec.return_value = enterprise_customer
                fake_get_sso_provider.return_value = 'test-sso-provider-id'

                assert handle_enterprise_logistration(backend, self.user) is None
                customer_sso_integration_config.refresh_from_db()
                assert customer_sso_integration_config.validated_at is not None



# Generated by Django 3.2.15 on 2023-01-03 14:48

from django.db import connection, migrations


def generate_index_sql(db_engine):
    if 'mysql' in db_engine:
        statement = """CREATE INDEX moodle_mldta_85936b55_idx
            ON moodle_moodlelearnerdatatransmissionaudit (enterprise_customer_uuid, plugin_configuration_id)
            ALGORITHM = INPLACE LOCK = NONE"""
        return statement
    elif 'postgresql' in db_engine:
        statement = """CREATE INDEX moodle_mldta_85936b55_idx
            ON moodle_moodlelearnerdatatransmissionaudit (enterprise_customer_uuid, plugin_configuration_id)"""
        return statement
    else:
        # Handle other database engines (e.g., SQLite)
        statement = """CREATE INDEX moodle_mldta_85936b55_idx
            ON moodle_moodlelearnerdatatransmissionaudit (enterprise_customer_uuid, plugin_configuration_id)"""
        return statement


def drop_index_sql(db_engine):
    return """DROP INDEX moodle_mldta_85936b55_idx
        ON moodle_moodlelearnerdatatransmissionaudit"""


class Migration(migrations.Migration):
    dependencies = [
        ('moodle', '0022_auto_20221220_1527'),
    ]

    # Determine the database engine
    db_engine = connection.settings_dict['ENGINE']

    operations = []

    if 'sqlite3' in db_engine:
        operations.append(
            migrations.AlterIndexTogether(
                name='moodlelearnerdatatransmissionaudit',
                index_together={('enterprise_customer_uuid', 'plugin_configuration_id')},
            )
        )
    else:
        operations.append(
            migrations.SeparateDatabaseAndState(
                state_operations=[
                    migrations.AlterIndexTogether(
                        name='moodlelearnerdatatransmissionaudit',
                        index_together={('enterprise_customer_uuid', 'plugin_configuration_id')},
                    ),
                ],
                database_operations=[
                    # Adjust SQL for MySQL and PostgreSQL
                    migrations.RunSQL(
                        sql=generate_index_sql(db_engine),
                        reverse_sql=drop_index_sql(db_engine),
                    ),
                ]
            )
        )



"""
Views for `EnterpriseCustomerAdmin` model.
"""
from edx_rbac.decorators import permission_required
from rest_framework import mixins, status, viewsets
from rest_framework.decorators import action
from rest_framework.pagination import PageNumberPagination
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from django.contrib.auth import get_user_model
from django.shortcuts import get_object_or_404

from enterprise import models, roles_api
from enterprise.api.v1.serializers import EnterpriseCustomerAdminSerializer
from enterprise.constants import ENTERPRISE_ADMIN_ROLE, ENTERPRISE_CUSTOMER_PROVISIONING_ADMIN_ACCESS_PERMISSION

User = get_user_model()


class EnterpriseCustomerAdminPagination(PageNumberPagination):
    """
    Pagination class for EnterpriseCustomerAdmin viewset.
    """
    page_size = 10
    page_size_query_param = 'page_size'
    max_page_size = 100


class EnterpriseCustomerAdminViewSet(
    mixins.ListModelMixin,
    mixins.UpdateModelMixin,
    mixins.RetrieveModelMixin,
    viewsets.GenericViewSet,
):
    """
    API views for the ``enterprise-customer-admin`` API endpoint.
    Only allows GET, and PATCH requests.
    """
    queryset = models.EnterpriseCustomerAdmin.objects.all()
    serializer_class = EnterpriseCustomerAdminSerializer
    permission_classes = (IsAuthenticated,)
    pagination_class = EnterpriseCustomerAdminPagination

    def get_queryset(self):
        """
        Filter queryset to only show records for the admin user.
        Excludes admins whose EnterpriseCustomerUser has been deactivated.
        """
        return models.EnterpriseCustomerAdmin.objects.filter(
            enterprise_customer_user__user_fk=self.request.user,
            enterprise_customer_user__active=True,
        )

    @action(detail=True, methods=['post'])
    def complete_tour_flow(self, request, pk=None):  # pylint: disable=unused-argument
        """
        Add a completed tour flow to the admin's completed_tour_flows.
        POST /api/v1/enterprise-customer-admin/{pk}/complete_tour_flow/

        Request Arguments:
        - ``flow_uuid``: The request object containing the flow_uuid

        Returns: A response indicating success or failure
        """
        admin = self.get_object()
        flow_uuid = request.data.get('flow_uuid')

        if not flow_uuid:
            return Response(
                {'error': 'flow_uuid is required'},
                status=status.HTTP_400_BAD_REQUEST
            )

        try:
            flow = get_object_or_404(models.OnboardingFlow, uuid=flow_uuid)
            admin.completed_tour_flows.add(flow)

            return Response({
                'status': 'success',
                'message': f'Successfully added tour flow {flow.title} to completed flows'
            })

        except models.OnboardingFlow.DoesNotExist:
            return Response(
                {'error': f'OnboardingFlow with uuid {flow_uuid} does not exist'},
                status=status.HTTP_404_NOT_FOUND
            )

    @permission_required(
        ENTERPRISE_CUSTOMER_PROVISIONING_ADMIN_ACCESS_PERMISSION,
        fn=lambda request, *args, **kwargs: request.data.get('enterprise_customer_uuid'),
    )
    @action(detail=False, methods=['post'])
    def create_admin_by_email(self, request):
        """
        Create a new EnterpriseCustomerAdmin record based on an email address.
        The email address must match an existing user.

        POST /api/v1/enterprise-customer-admin/create_admin_by_email/

        The requesting user must have the ``enterprise_provisioning_admin``
        role to access this endpoint.

        Request Arguments:
        - ``email``: Email address of the user to make an admin
        - ``enterprise_customer_uuid``: UUID of the enterprise customer

        Returns: A response with the created admin record.
        """
        email = request.data.get('email')
        enterprise_customer_uuid = request.data.get('enterprise_customer_uuid')

        if not email:
            return Response(
                {'error': 'email is required'}, status=status.HTTP_400_BAD_REQUEST,
            )

        if not enterprise_customer_uuid:
            return Response(
                {'error': 'enterprise_customer_uuid is required'}, status=status.HTTP_400_BAD_REQUEST,
            )

        try:
            user = User.objects.get(email=email)
        except User.DoesNotExist:
            return Response(
                {'error': f'User with email {email} does not exist'}, status=status.HTTP_404_NOT_FOUND,
            )

        try:
            enterprise_customer = models.EnterpriseCustomer.objects.get(uuid=enterprise_customer_uuid)
        except models.EnterpriseCustomer.DoesNotExist:
            return Response(
                {'error': f'EnterpriseCustomer with uuid {enterprise_customer_uuid} does not exist'},
                status=status.HTTP_404_NOT_FOUND
            )

        # Get or create EnterpriseCustomerUser
        enterprise_customer_user, _ = models.EnterpriseCustomerUser.objects.get_or_create(
            enterprise_customer=enterprise_customer,
            user_fk=user,
            defaults={'user_id': user.id}
        )

        response_status_code = status.HTTP_200_OK
        admin, was_created = models.EnterpriseCustomerAdmin.objects.get_or_create(
            enterprise_customer_user=enterprise_customer_user
        )
        if was_created:
            response_status_code = status.HTTP_201_CREATED

        roles_api.assign_admin_role(
            enterprise_customer_user.user,
            enterprise_customer=enterprise_customer_user.enterprise_customer
        )

        serializer = self.get_serializer(admin)
        return Response(serializer.data, status=response_status_code)

    @permission_required(
        ENTERPRISE_CUSTOMER_PROVISIONING_ADMIN_ACCESS_PERMISSION,
        fn=lambda request, enterprise_customer_uuid, *args, **kwargs: enterprise_customer_uuid,
    )
    def delete_admin(self, request, enterprise_customer_uuid=None, admin_pk=None):
        """
        Soft delete an EnterpriseCustomerAdmin record.
        DELETE /api/v1/enterprise-customer/{enterprise_customer_uuid}/admins/{admin_pk}/

        The requesting user must have the ``enterprise_provisioning_admin``
        role to access this endpoint.

        Removes the enterprise_admin role assignment and deactivates the
        EnterpriseCustomerUser if the user has no other roles for the enterprise.
        The ECA record itself is left untouched in the database.
        """
        # Validate enterprise customer
        try:
            enterprise_customer = models.EnterpriseCustomer.objects.get(uuid=enterprise_customer_uuid)
        except models.EnterpriseCustomer.DoesNotExist:
            return Response(
                {'error': f'EnterpriseCustomer with uuid {enterprise_customer_uuid} does not exist'},
                status=status.HTTP_404_NOT_FOUND,
            )

        # Look up admin record
        try:
            admin = models.EnterpriseCustomerAdmin.objects.get(pk=admin_pk)
        except models.EnterpriseCustomerAdmin.DoesNotExist:
            return Response(
                {'error': f'EnterpriseCustomerAdmin with id {admin_pk} does not exist'},
                status=status.HTTP_404_NOT_FOUND,
            )

        # Verify admin belongs to the given enterprise customer
        if admin.enterprise_customer_user.enterprise_customer_id != enterprise_customer.uuid:
            return Response(
                {'error': 'Admin does not belong to the specified enterprise customer'},
                status=status.HTTP_404_NOT_FOUND,
            )

        # Remove enterprise_admin role
        enterprise_customer_user = admin.enterprise_customer_user
        user = enterprise_customer_user.user
        roles_api.delete_admin_role_assignment(user=user, enterprise_customer=enterprise_customer)

        # Check if user has other roles for this enterprise
        has_other_roles = models.SystemWideEnterpriseUserRoleAssignment.objects.filter(
            user=user,
            enterprise_customer=enterprise_customer,
        ).exclude(
            role__name=ENTERPRISE_ADMIN_ROLE,
        ).exists()

        # If no other roles, deactivate the EnterpriseCustomerUser
        if not has_other_roles:
            enterprise_customer_user.active = False
            enterprise_customer_user.save(update_fields=['active', 'modified'])

        return Response(status=status.HTTP_200_OK)



"""
Utils for interacting with cache interfaces.
"""
import hashlib

from django.conf import settings

from enterprise import __version__ as code_version

CACHE_KEY_SEP = ':'
DEFAULT_NAMESPACE = 'edx-enterprise-default'


def versioned_cache_key(*args):
    """
    Utility to produce a versioned cache key, which includes
    an optional settings variable and the current code version,
    so that we can perform key-based cache invalidation.
    """
    components = [str(arg) for arg in args]
    components.append(code_version)
    if stamp_from_settings := getattr(settings, 'ENTERPRISE_CACHE_KEY_VERSION_STAMP', None):
        components.append(stamp_from_settings)
    decoded_cache_key = CACHE_KEY_SEP.join(components)
    return hashlib.sha512(decoded_cache_key.encode()).hexdigest()
